==> 0.9.7.tar.gz: lib/src/parsers/yaml/parser.dart <==
import 'package:localization_builder/src/definitions/case.dart';
import 'package:localization_builder/src/definitions/condition.dart';
import 'package:localization_builder/src/definitions/label.dart';
import 'package:localization_builder/src/definitions/localizations.dart';
import 'package:localization_builder/src/definitions/section.dart';
import 'package:localization_builder/src/parsers/result.dart';
import 'package:yaml/yaml.dart';

import '../parser.dart';
import 'token.dart';

class YamlLocalizationParser
    extends LocalizationParser<YamlNode, YamlLocalizationToken> {
  YamlLocalizationParser();

  List<YamlLocalizationToken> _tokens;

  @override
  ParsingResult<YamlNode, YamlLocalizationToken> parse(YamlNode input,
      {String name}) {
    if (input == null) {
      return ParsingResult<YamlNode, YamlLocalizationToken>(
        input: input,
        result: Localizations(
          name: name,
          supportedLanguageCodes: const <String>[],
          children: const <Section>[],
          labels: const <Label>[],
        ),
        tokens: const <YamlLocalizationToken>[],
      );
    }

    if (!(input is YamlMap)) {
      throw ParsingException<YamlLocalizationToken>(
        message: '[input] should be a YamlMap',
        token: YamlLocalizationToken.unknown(input),
      );
    }
    final map = input as YamlMap;
    _tokens = <YamlLocalizationToken>[];

    // Parsing language coedes
    final languages = <Localizations>[];

    for (var languageEntry in map.nodes.entries) {
      final language = Localizations(
        name: name,
        supportedLanguageCodes: [languageEntry.key.toString()],
      );

      _addToken(
        YamlLocalizationTokenType.languageKey,
        languageEntry.key,
      );
      if (!(languageEntry.value is YamlMap)) {
        throw ParsingException<YamlLocalizationToken>(
          message: '[${languageEntry.key}] should be a YamlMap',
          token: YamlLocalizationToken.unknown(languageEntry.value),
        );
      }
      _parseSection(
        languageEntry.key.toString(),
        language,
        languageEntry.value,
      );
      languages.add(language);
    }

    return ParsingResult<YamlNode, YamlLocalizationToken>(
      input: input,
      result: Localizations.merge(name, languages),
      tokens: _tokens,
    );
  }

  void _parseSection(String languageKey, Section section, YamlMap map) {
    for (var sectionEntry in map.nodes.entries) {
      final value = sectionEntry.value;
      // Label
      if (value is YamlScalar) {
        final labelTranslation = value.value.toString();
        _addToken(
          YamlLocalizationTokenType.labelKey,
          sectionEntry.key,
        );
        _addToken(
          YamlLocalizationTokenType.labelValue,
          sectionEntry.value,
        );
        section.labels.add(
          Label(
            key: sectionEntry.key.toString(),
            cases: <Case>[
              Case(
                condition: DefaultCondition(),
                translations: [
                  Translation(
                    languageKey,
                    labelTranslation,
                  ),
                ],
              ),
            ],
          ),
        );
      } else if (value is YamlMap) {
        // Label with cases
        if (value.containsKey('cases')) {
          _addToken(
            YamlLocalizationTokenType.labelKey,
            sectionEntry.key,
          );
          final cases = <Case>[];

          final caseNodes = value.nodes['cases'];

          if (!(caseNodes is YamlMap)) {
            throw ParsingException<YamlLocalizationToken>(
              message: '[cases] should be a YamlMap',
              token: YamlLocalizationToken.unknown(caseNodes),
            );
          }

          _addToken(
            YamlLocalizationTokenType.caseKey,
            value.nodes.entries
                .firstWhere((x) => x.key.toString() == 'cases')
                .key,
          );

          final caseMap = caseNodes as YamlMap;

          for (var caseEntry in caseMap.nodes.entries) {
            _addToken(
              YamlLocalizationTokenType.caseKey,
              caseEntry.key,
            );
            _addToken(
              YamlLocalizationTokenType.caseValue,
              caseEntry.value,
            );
            final caseTranslation = caseEntry.value.toString();
            final condition = Condition.parse(caseEntry.key.toString());
            cases.add(
              Case(
                condition: condition,
                translations: [
                  Translation(
                    languageKey,
                    caseTranslation,
                  ),
                ],
              ),
            );
          }

          section.labels.add(
            Label(
              key: sectionEntry.key.toString(),
              cases: cases,
            ),
          );
        }
        // Child section
        else {
          _addToken(
            YamlLocalizationTokenType.sectionKey,
            sectionEntry.key,
          );
          final sectionKey = sectionEntry.key.toString();
          print('S: $sectionKey');
          final child = Section(
            key: sectionKey,
            path: <String>[
              ...section.path,
              sectionKey,
            ],
          );
          section.children.add(child);

          if (!(sectionEntry.value is YamlMap)) {
            throw ParsingException<YamlLocalizationToken>(
              message: '[$sectionKey] should be a YamlMap',
              token: YamlLocalizationToken.unknown(sectionEntry.value),
            );
          }

          _parseSection(languageKey, child, sectionEntry.value);
        }
      }
    }
  }

  void _addToken(YamlLocalizationTokenType type, YamlNode node) {
    _tokens.add(
      YamlLocalizationToken(
        type: type,
        node: node,
      ),
    );
  }
}

==> 0.11.6.tar.gz: lib/src/parsers/yaml/parser.dart <==
import 'package:localization_builder/src/definitions/case.dart';
import 'package:localization_builder/src/definitions/condition.dart';
import 'package:localization_builder/src/definitions/label.dart';
import 'package:localization_builder/src/definitions/localizations.dart';
import 'package:localization_builder/src/definitions/section.dart';
import 'package:localization_builder/src/definitions/translation.dart';
import 'package:localization_builder/src/parsers/result.dart';
import 'package:yaml/yaml.dart';

import '../parser.dart';
import 'token.dart';

class YamlLocalizationParser
    extends LocalizationParser<YamlNode, YamlLocalizationToken> {
  YamlLocalizationParser();

  List<YamlLocalizationToken> _tokens = <YamlLocalizationToken>[];

  @override
  ParsingResult<YamlNode, YamlLocalizationToken> parse({
    required YamlNode input,
    required String name,
  }) {
    if (!(input is YamlMap)) {
      throw ParsingException<YamlLocalizationToken>(
        message: '[input] should be a YamlMap',
        token: YamlLocalizationToken.unknown(input),
      );
    }
    final map = input;
    _tokens = <YamlLocalizationToken>[];

    // Parsing language coedes
    final languages = <Localizations>[];

    for (var languageEntry in map.nodes.entries) {
      _addToken(
        YamlLocalizationTokenType.languageKey,
        languageEntry.key,
      );
      if (!(languageEntry.value is YamlMap)) {
        throw ParsingException<YamlLocalizationToken>(
          message: '[${languageEntry.key}] should be a YamlMap',
          token: YamlLocalizationToken.unknown(languageEntry.value),
        );
      }
      final section = _parseSection(
        languageEntry.key.toString(),
        name,
        languageEntry.value as YamlMap,
      );
      languages.add(
        Localizations.fromSection(
          name: name,
          supportedLanguageCodes: [languageEntry.key.toString()],
          section: section,
        ),
      );
    }

    return ParsingResult<YamlNode, YamlLocalizationToken>(
      input: input,
      result: Localizations.merge(name, languages),
      tokens: _tokens,
    );
  }

  Section _parseSection(String languageKey, String key, YamlMap map) {
    final labels = <Label>[];
    final children = <Section>[];
    for (var sectionEntry in map.nodes.entries) {
      final value = sectionEntry.value;
      // Label
      if (value is YamlScalar) {
        final labelTranslation = value.value.toString();
        _addToken(
          YamlLocalizationTokenType.labelKey,
          sectionEntry.key,
        );
        _addToken(
          YamlLocalizationTokenType.labelValue,
          sectionEntry.value,
        );
        labels.add(
          Label(
            key: sectionEntry.key.toString(),
            cases: <Case>[
              Case(
                condition: DefaultCondition(),
                translations: [
                  Translation(
                    languageKey,
                    labelTranslation,
                  ),
                ],
              ),
            ],
          ),
        );
      } else if (value is YamlMap) {
        // Label with cases
        if (value.containsKey('cases')) {
          _addToken(
            YamlLocalizationTokenType.labelKey,
            sectionEntry.key,
          );
          final cases = <Case>[];

          final caseNodes = value.nodes['cases'];

          if (!(caseNodes is YamlMap)) {
            throw ParsingException<YamlLocalizationToken>(
              message: '[cases] should be a YamlMap',
              token: YamlLocalizationToken.unknown(caseNodes),
            );
          }

          _addToken(
            YamlLocalizationTokenType.caseKey,
            value.nodes.entries
                .firstWhere((x) => x.key.toString() == 'cases')
                .key,
          );

          final caseMap = caseNodes;

          for (var caseEntry in caseMap.nodes.entries) {
            _addToken(
              YamlLocalizationTokenType.caseKey,
              caseEntry.key,
            );
            _addToken(
              YamlLocalizationTokenType.caseValue,
              caseEntry.value,
            );
            final caseTranslation = caseEntry.value.toString();
            final condition = Condition.parse(caseEntry.key.toString());
            cases.add(
              Case(
                condition: condition,
                translations: [
                  Translation(
                    languageKey,
                    caseTranslation,
                  ),
                ],
              ),
            );
          }

          labels.add(
            Label(
              key: sectionEntry.key.toString(),
              cases: cases,
            ),
          );
        }
        // Child section
        else {
          _addToken(
            YamlLocalizationTokenType.sectionKey,
            sectionEntry.key,
          );
          final sectionKey = sectionEntry.key.toString();
          print('S: $sectionKey');

          if (!(sectionEntry.value is YamlMap)) {
            throw ParsingException<YamlLocalizationToken>(
              message: '[$sectionKey] should be a YamlMap',
              token: YamlLocalizationToken.unknown(sectionEntry.value),
            );
          }

          final child = _parseSection(
            languageKey,
            sectionKey,
            sectionEntry.value as YamlMap,
          );
          children.add(child);
        }
      }
    }
    return Section(
      key: key,
      children: children,
      labels: labels,
    );
  }

  void _addToken(YamlLocalizationTokenType type, YamlNode node) {
    _tokens.add(
      YamlLocalizationToken(
        type: type,
        node: node,
      ),
    );
  }
}
